#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
#
# Name: WebRcon (webrcon.pl)
# Auth: dumpsterKEEPER
# Date: 04-18-03
# Desc: A web-based tool that allows rcon administration for Q3 engine games
#
# THIS CODE IS PROVIDED AS IS. THE AUTHOR MAKES NO GUARANTEES AS PERTAINING TO
# THE RELIABILITY OR SECURITY OF THIS CODE. YOU USE IT AT YOUR OWN RISK. THE
# AUTHOR IS NOT LIABLE FOR ANY DAMAGE OR LOSS RESULTING FROM USING THIS CODE.
#
# THIS CODE CAN BE FREELY DISTRIBUTED WITH A FEW CONDITIONS BEING MET. FIRST,
# PROPER CREDIT MUST ALWAYS BE GIVEN TO THE ORIGINAL AUTHOR. THE AUTHOR NOTICE
# AND CONTACT INFORMATION MUST ALWAYS REMAIN WITH THE CODE; IT CANNOT BE
# REMOVED FOR ANY REASON. SECOND, IF YOU SHOULD MODIFY ANY PORTION OF THIS
# CODE, THE MODIFIED VERSION CANNOT BE DISTRIBUTED WITHOUT THE ORIGINAL
# AUTHOR'S CONSENT.
# -----------------------------------------------------------------------------
use strict;
use CGI qw(:standard);
use Socket;

# -----------------------------------------------------------------------------
# *** SCRIPT CONFIGURATION ***
# -----------------------------------------------------------------------------
my $HOSTNAME = "192.168.0.1";  # Address or hostname of server (recommended)
my $PORTNUM  = "20100";        # Port server is listening on (recommended)
my $PASSWORD = "";             # Rcon password (not recommended)

my @SRVCOMMANDS = qw(status serverinfo mapcycle);
      # Server quick access command buttons
my @PBCOMMANDS  = qw(pb_sv_ver pb_sv_banlist pb_sv_update);
      # PunkBuster quick access command buttons

# Flags
my $NOT24HOURTIME   = "1"; # Flag to set 12 or 24-hour time
my $STRIPCOLORCODES = "1"; # Remove color codes from text (i.e. ^1, ^2, etc)

# HTML Configuration
my $BGCOLOR      = "#2a2c40";           # Page background
my $BGIMAGE      = "";                  # Background image (optional)
my $FGCOLOR      = "#ffffff";           # Font color
my $FONTFACE     = "Arial, sans-serif"; # Font for general text
my $RESPONSEFONT = "Courier New";       # Font for server response (fixed width)
my $TBLBGCOLOR   = "#2a2c40";           # Background for tables
my $LINKCOLOR    = "#ffffff";           # L/A/V link color
# -----------------------------------------------------------------------------
# *** END SCRIPT CONFIGURATION ***
#
# *** DO NOT MODIFY BELOW THIS POINT ***
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# PROGRAM CONSTANTS
# -----------------------------------------------------------------------------
my $INTROSTR  = chr(255) . chr(255) . chr(255) . chr(255);
                      # Character garbage in server response lines
my $AM        = "AM"; # Morning constant for 12-hour time
my $PM        = "PM"; # Evening constant for 12-hour time

my $AUTH      = "Yourname";              # Author name
my $AUTHEMAIL = "you@wherever.com";      # Author email
my $NAME      = "WebRcon";               # Name of program
my $VERSION   = "1.1";                   # Current program version (major.minor)

# -----------------------------------------------------------------------------
# PROGRAM VARIABLES
# -----------------------------------------------------------------------------
my $HostName    = param("address");  # Server hostname
my $PortNum     = param("port");     # Server port number
my $Password    = param("password"); # Server password
my $Cmd         = param("cmd");      # Rcon command
my $PortAddr    = "";                # Specific address to send command to
my $SrvResponse = "";                # Complete response from server

# -----------------------------------------------------------------------------
# Main Program
# Last Modified: 4/24/2003 8:00PM
# -----------------------------------------------------------------------------
DisplayPageHeader();
if ($Cmd)
{
   if ($HOSTNAME) { $HostName = $HOSTNAME; }
   if ($PORTNUM)  { $PortNum  = $PORTNUM;  }
   if ($PASSWORD) { $Password = $PASSWORD; }

   SendSrvCommand();
   FormatResponse();
}
DisplayCmdForm();
if ($Cmd)
{
   DisplayHTMLResponse();
}
DisplayPageFooter();

# -----------------------------------------------------------------------------
# Writes the header of the HTML output page
# Last Modified: 4/24/2003 9:25PM
# -----------------------------------------------------------------------------
sub DisplayPageHeader
{
   print header;
   print "<html><head><title>$NAME v$VERSION</title>\n";
   print "<meta http-equiv=\"pragma\" content=\"no-cache\">\n";
   print "<meta http-equiv=\"expires\" content=\"-1\">\n";
   print "<script language=\"JavaScript\">\n";
   if ($Cmd)
   {
      print "function SetFocus() { document.command.cmd.select(); }\n";
   }
   else
   {
      print "function SetFocus() { document.command.elements[0].focus(); }\n";
   }
   print "window.onload = SetFocus;</script>\n";
   print "</head><body background=\"$BGIMAGE\" bgcolor=\"$BGCOLOR\" link=\"$LINKCOLOR\" ";
   print "alink=\"$LINKCOLOR\" vlink=\"$LINKCOLOR\" text=\"$FGCOLOR\" ";
   print "topmargin=\"0\" bottommargin=\"0\" rightmargin=\"0\" leftmargin=\"0\">\n";

   print "<table border=\"0\" cellpadding=\"5\" cellspacing=\"0\" width=\"100%\">

   print Tr(
         td({-colspan=>'3',
                   -width=>'100%'},
                   "<font face=\"$FONTFACE\" size=\"4\"><b><center>$NAME v$VERSION</center></b>\n",
                   "</font>"),
      );
}

# -----------------------------------------------------------------------------
# Send rcon command to server and listen for response
# Last Modified: 4/28/2003 6:33PM
# -----------------------------------------------------------------------------
sub SendSrvCommand
{
   my $Msg;           # Text string to send to server
   my $Response = ""; # Holds individual response lines from server

   # Build message to send to server
   $Msg = $INTROSTR . "rcon $Password $Cmd";

   # Send message
   socket(SOCKET, PF_INET, SOCK_DGRAM, getprotobyname("udp")) || Bail("\nCannot create socket: $!");
   $PortAddr = sockaddr_in($PortNum, inet_aton($HostName));
   defined(send(SOCKET, $Msg, 0, $PortAddr)) || Bail("\nError sending message: $!");

   while (1)
   {
      # Receive response
      defined($PortAddr = recv(SOCKET, $Response, 1024, 0)) || Bail("\nError receiving response: $!");

      # Remove garbage from response
      $Response =~ s/($INTROSTR)print\n//;
      $Response =~ s/^\n//;
      last if ($Response eq ""); # Check for end of response

      $SrvResponse .= $Response;
   }

   close(SOCKET);
}

# -----------------------------------------------------------------------------
# Format the server response text
# Last Modified: 5/28/2003 8:18PM
# -----------------------------------------------------------------------------
sub FormatResponse
{
   # Convert ASCII characters to HTML
   $SrvResponse =~ s/</\&lt\;/g;     # Left bracket
   $SrvResponse =~ s/>/\&gt\;/g;     # Right bracket
   $SrvResponse =~ s/  / \&nbsp\;/g; # Spaces
   $SrvResponse =~ s/\n/<br \/>/g;   # Newline

   if ($STRIPCOLORCODES)
   {
      # Remove color codes (i.e. ^1, ^2, etc)
      $SrvResponse =~ s/\^.+?//g;
   }
}

# -----------------------------------------------------------------------------
# Displays the form for sending rcon commands
# Last Modified: 5/28/2003 8:07PM
# -----------------------------------------------------------------------------
sub DisplayCmdForm
{
   print start_form({-name=>'command'});
   print "<br /><table bgcolor=\"$TBLBGCOLOR\" border=\"1\" bordercolor=\"$FGCOLOR\" cellpadding=\"15\" cellspacing=\"0\" width=\"90%\" align=\"center\">\n";

   print "<tr>";
   if ($HOSTNAME) # Display address constant
   {
      print td({-width=>'33%'},
                      "<font face=\"$FONTFACE\" size=\"3\"><b>Server Address:</b></font><br>",
                      "<font face=\"$FONTFACE\" size=\"2\">$HOSTNAME</font>\n");
   }
   else # Display textfield
   {
      print td("<font face=\"$FONTFACE\" size=\"3\"><b>Server Address:</b>\n",
               br,
               textfield({-name=>'address',
                          -size=>'25',
                          -maxlength=>'25'}),
               "</font>");
   }

   if ($PORTNUM) # Display port number constant
   {
      print td({-width=>'33%'},
                      "<font face=\"$FONTFACE\" size=\"3\"><b>Server Port:</b></font><br>",
                      "<font face=\"$FONTFACE\" size=\"2\">$PORTNUM</font>\n");
   }
   else # Display textfield
   {
      print td("<font face=\"$FONTFACE\" size=\"3\"><b>Server Port:</b>\n",
               br,
               textfield({-name=>'port',
                          -size=>'25',
                          -maxlength=>'6'}),
               "</font>");
   }

   if ($PASSWORD) # Display password constant
   {
      my $Asterisks = "";

      # Mask real password with asterisks for display purposes
      while ($PASSWORD =~ /(.)/g) { $Asterisks .= "* "; }

      print td({-width=>'33%'},
                      "<font face=\"$FONTFACE\" size=\"3\"><b>Rcon Password:</b></font><br>",
                      "<font face=\"$FONTFACE\" size=\"2\">$Asterisks</font>\n");
   }
   else # Display textfield
   {
      print td("<font face=\"$FONTFACE\" size=\"3\"><b>Rcon Password:</b>\n",
               br,
               password_field({-name=>'password',
                               -size=>'25',
                               -maxlength=>'20'}),
               "</font>");
   }
   print "</tr>";

   print "<tr><td colspan=\"3\" width=\"100%\"><font face=\"$FONTFACE\" size=\"3\"><b>Rcon Command:</b>\n",
         br, textfield({-name=>'cmd', -size=>'60', -maxlength=>'255'}), "&nbsp;&nbsp;", submit({-value=>'Send Command'});

   DisplayQuickButtons();

   print "</font></td></tr></table>", end_form;
}

# -----------------------------------------------------------------------------
# Displays the quick access buttons for both server and PB commands
# Last Modified: 4/24/2003 8:00PM
# -----------------------------------------------------------------------------
sub DisplayQuickButtons
{
   print "<br><br>";
   foreach (@SRVCOMMANDS)
   {
      print "<input type=\"button\" name=\"$_\" value=\"$_\" ";
      print "onClick=\"document.command.cmd.value='$_'; document.command.submit(); return false\" />&nbsp;&nbsp;\n";
   }

   print "<br><br>";
   foreach (@PBCOMMANDS)
   {
      print "<input type=\"button\" name=\"$_\" value=\"$_\" ";
      print "onClick=\"document.command.cmd.value='$_'; document.command.submit(); return false\" />&nbsp;&nbsp;\n";
   }
}

# -----------------------------------------------------------------------------
# Displays the response from the server in HTML format
# Last Modified: 4/28/2003 6:41PM
# -----------------------------------------------------------------------------
sub DisplayHTMLResponse
{
   if ($SrvResponse)
   {
      print "<br /><table bgcolor=\"$TBLBGCOLOR\" border=\"1\" bordercolor=\"$FGCOLOR\" cellpadding=\"20\" cellspacing=\"0\" width=\"90%\" align=\"center\">\n";
      print "<tr><td><font face=\"$RESPONSEFONT\" size=\"2\">[" . GetDateTime() . "] <b>$Cmd:</b><br><br>$SrvResponse</font></td></tr>\n";
      print "</table>";
   }
   else # No response
   {
      print "<br /><table bgcolor=\"$TBLBGCOLOR\" border=\"1\" bordercolor=\"$FGCOLOR\" cellpadding=\"20\" cellspacing=\"0\" width=\"90%\" align=\"center\">\n";
      print "<tr><td><font face=\"$RESPONSEFONT\" size=\"2\">No response from server</font></td></tr>\n";
      print "</table>";
   }
}

# -----------------------------------------------------------------------------
# Writes the footer of the HTML output page
# Last Modified: 4/24/2003 9:19PM
# -----------------------------------------------------------------------------
sub DisplayPageFooter
{
   print "<br /><br /><table bgcolor=\"$TBLBGCOLOR\" border=\"1\" bordercolor=\"$FGCOLOR\" cellpadding=\"5\" cellspacing=\"0\" width=\"40%\" align=\"center\">\n";
   print Tr(
            td("<font face=\"$FONTFACE\" size=\"1\"><center>$NAME v$VERSION running under Perl v$]<br><br>",
                      "Brought to you by $AUTH (<a href=\"mailto:$AUTHEMAIL\"?subject=WebRcon>$AUTHEMAIL</a>)<br><br>",
                      "Comments, suggestions, and bug reports are appreciated</center></font>"));
   print "</table>", end_html;
}

# -----------------------------------------------------------------------------
# Get the current date/time and return it as a formatted string
# Last Modified: 4/28/2003 7:12PM
# -----------------------------------------------------------------------------
sub GetDateTime
{
   my $TimeofDay = ""; # Holds morning/evening constants for 12-hour time
   my ($min, $hours, $mday, $month, $year) = (localtime)[1..5];
                       # Individual date/time values

   $month++;
   $year = substr($year + 1900, 2); # Convert year to 2 digits
   if ($NOT24HOURTIME) # Convert 24-hour time to 12-hour time
   {
      # Set the time of day
      if ($hours >= 12)
      {
         $TimeofDay = " $PM";
      }
      else
      {
         $TimeofDay = " $AM";
      }

      # Convert the hour field
      if ($hours == 0)
      {
         $hours = 12;
      }
      elsif ($hours > 12)
      {
         $hours = $hours - 12;
      }
   }
   if ($hours < 10) { $hours = "0" . $hours }
   if ($min   < 10) { $min   = "0" . $min }
   if ($mday  < 10) { $mday  = "0" . $mday }
   if ($month < 10) { $month = "0" . $month }

   return "$month-$mday-$year $hours:$min$TimeofDay";
}

# -----------------------------------------------------------------------------
# A generalized error sub, prints the error and attempts to exit gracefully
# Last Modified: 4/28/2003 7:19PM
# -----------------------------------------------------------------------------
sub Bail
{
   my $Error = "@_"; # Text error message

   print "<br /><table bgcolor=\"$TBLBGCOLOR\" border=\"0\" cellpadding=\"10\" cellspacing=\"0\" width=\"50%\" align=\"center\">";
   print Tr(
            td("<font face=\"$FONTFACE\" size=\"4\">$NAME Error:</font>") );
   print Tr(
            td("<font face=\"$FONTFACE\" size=\"2\">$Error<br /><br /><br />",
                      "If this problem persists, please contact your server administrator</font>") );
   print "</table>";

   DisplayPageFooter();

   exit(-1);
}
